/// <reference types="node" />
import { EventEmitter } from "events";
import StrictEmitter from "strict-event-emitter-types";
export declare class PromiseQueue extends EventEmitter implements StrictEmitter<EventEmitter, {
    error: (error: Error) => void;
}> {
    #private;
    /** The number of tasks that will be executed concurrently.
     * Note that if concurrency > 1, the queue cannot guarantee FIFO since the tasks' execution times may vary. */
    readonly concurrency: number;
    /** Number of tasks currently in the queue and awaiting completion. */
    size: number;
    /** Flag indicating the queue is currently processing tasks. */
    PAUSE: boolean;
    constructor(
    /** The number of tasks that will be executed concurrently.
     * Note that if concurrency > 1, the queue cannot guarantee FIFO since the tasks' execution times may vary. */
    concurrency?: number);
    /** Pause queue execution */
    pause(): void;
    /** Resume queue execution */
    resume(): void;
    /** Add a prioritized task to the queue.
     * Note that a lower priority number means that the task will be prioritized.
     * Ie.: `0` is the highest priority value.
     */
    addTask<T extends (...args: any[]) => Promise<unknown>>(priority: number, task: T): Promise<T extends (...args: any[]) => Promise<infer r> ? r : never>;
}
