"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var _PromiseQueue_instances, _PromiseQueue_inflight, _PromiseQueue_bucketQueue, _PromiseQueue_emitter, _PromiseQueue_processTasks;
Object.defineProperty(exports, "__esModule", { value: true });
exports.PromiseQueue = void 0;
const assert = require("assert");
const events_1 = require("events");
class PromiseQueue extends events_1.EventEmitter {
    constructor(
    /** The number of tasks that will be executed concurrently.
     * Note that if concurrency > 1, the queue cannot guarantee FIFO since the tasks' execution times may vary. */
    concurrency = 1) {
        super();
        this.concurrency = concurrency;
        _PromiseQueue_instances.add(this);
        /** Number of tasks currently in the queue and awaiting completion. */
        this.size = 0;
        /** Flag indicating the queue is currently processing tasks. */
        this.PAUSE = false;
        /** Number of tasks currently being executed. */
        _PromiseQueue_inflight.set(this, 0);
        _PromiseQueue_bucketQueue.set(this, []);
        /** Internal signalling. Used to trigger queue processing. */
        _PromiseQueue_emitter.set(this, new events_1.EventEmitter());
        assert(concurrency > 0, "'concurrency' must be greater than 0");
        __classPrivateFieldGet(this, _PromiseQueue_emitter, "f").on("new-task", __classPrivateFieldGet(this, _PromiseQueue_instances, "m", _PromiseQueue_processTasks).bind(this));
        __classPrivateFieldGet(this, _PromiseQueue_emitter, "f").on("complete", __classPrivateFieldGet(this, _PromiseQueue_instances, "m", _PromiseQueue_processTasks).bind(this));
        __classPrivateFieldGet(this, _PromiseQueue_emitter, "f").on("resume", () => {
            // The queue may process multiple tasks concurrently upon resuming.
            const availableCapacity = this.concurrency - __classPrivateFieldGet(this, _PromiseQueue_inflight, "f");
            for (let i = 0; i < availableCapacity; i++) {
                void __classPrivateFieldGet(this, _PromiseQueue_instances, "m", _PromiseQueue_processTasks).call(this);
            }
        });
    }
    /** Pause queue execution */
    pause() {
        this.PAUSE = true;
    }
    /** Resume queue execution */
    resume() {
        this.PAUSE = false;
        __classPrivateFieldGet(this, _PromiseQueue_emitter, "f").emit("resume");
    }
    /** Add a prioritized task to the queue.
     * Note that a lower priority number means that the task will be prioritized.
     * Ie.: `0` is the highest priority value.
     */
    async addTask(priority, task) {
        assert(priority >= 0, "Priority must be >= 0.");
        // If the priority references a bucket that hasn't been initialized we must
        // initialize all the buckets at indices 0 -> priority.
        if (priority >= __classPrivateFieldGet(this, _PromiseQueue_bucketQueue, "f").length) {
            const fillers = new Array(priority - __classPrivateFieldGet(this, _PromiseQueue_bucketQueue, "f").length + 1)
                .fill(0)
                .map(() => []);
            __classPrivateFieldGet(this, _PromiseQueue_bucketQueue, "f").push(...fillers);
        }
        return await new Promise((resolve, reject) => {
            __classPrivateFieldGet(this, _PromiseQueue_bucketQueue, "f")[priority].push({ task, resolve, reject });
            this.size += 1; // total queue size. Decreased within the async iterator.
            __classPrivateFieldGet(this, _PromiseQueue_emitter, "f").emit("new-task");
        });
    }
}
exports.PromiseQueue = PromiseQueue;
_PromiseQueue_inflight = new WeakMap(), _PromiseQueue_bucketQueue = new WeakMap(), _PromiseQueue_emitter = new WeakMap(), _PromiseQueue_instances = new WeakSet(), _PromiseQueue_processTasks = 
/** Pick and execute the highest priority task. */
async function _PromiseQueue_processTasks() {
    try {
        if (this.PAUSE || __classPrivateFieldGet(this, _PromiseQueue_inflight, "f") >= this.concurrency || this.size === 0) {
            return;
        }
        // Find the lowest index (ie highest priority) task.
        const priorityIndex = __classPrivateFieldGet(this, _PromiseQueue_bucketQueue, "f").findIndex(entries => entries.length);
        assert(priorityIndex >= 0, `Expected the queue to contain entries since the queue size is: ${this.size}`);
        __classPrivateFieldSet(this, _PromiseQueue_inflight, __classPrivateFieldGet(this, _PromiseQueue_inflight, "f") + 1, "f");
        const priorityItems = __classPrivateFieldGet(this, _PromiseQueue_bucketQueue, "f")[priorityIndex];
        const nextTask = priorityItems.shift();
        this.size -= 1;
        assert(nextTask, "Expected the item list to contain an entry since findIndex() returned != -1");
        const { task, resolve, reject } = nextTask;
        try {
            resolve(await task());
        }
        catch (error) {
            reject(error);
        }
        __classPrivateFieldSet(this, _PromiseQueue_inflight, __classPrivateFieldGet(this, _PromiseQueue_inflight, "f") - 1, "f");
        __classPrivateFieldGet(this, _PromiseQueue_emitter, "f").emit("complete");
    }
    catch (error) {
        this.emit("error", error);
    }
};
